package cnpc.core.common.utils;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import javax.net.ssl.HttpsURLConnection;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;

public class HttpUtil {
    private static final Log logger = LogFactory.getLog(HttpUtil.class);

    public static String charset = "utf-8";


    public static String requestPost(String url, List<NameValuePair> params) throws ClientProtocolException, IOException {
        CloseableHttpClient httpclient = HttpClientBuilder.create().build();
        HttpPost httppost = new HttpPost(url);
        httppost.setEntity(new UrlEncodedFormEntity(params, "utf-8"));
        CloseableHttpResponse response = httpclient.execute(httppost);
        System.out.println(response.toString());

        HttpEntity entity = response.getEntity();
        String jsonStr = EntityUtils.toString(entity, "utf-8");
        System.out.println(jsonStr);
        httppost.releaseConnection();
        return jsonStr;
    }

    public static String sendGet(String url, String param, Map<String, String> reqHead) {
        return sendGet(url, param, reqHead, charset);
    }


    public static String sendGet(String url, String param, Map<String, String> reqHead, String charset) {
        StringBuffer result = new StringBuffer();
        BufferedReader in = null;
        try {
            InputStream inputStream = HttpUtil.sendGetInputStream(url, param, reqHead);
            // 定义 BufferedReader输入流来读取URL的响应
            in = new BufferedReader(new InputStreamReader(inputStream, charset));
            String line;
            while ((line = in.readLine()) != null) {
                line = line.trim();
                result.append(line);
            }
        } catch (Exception e) {
            logger.error("发送GET请求出现异常！" + url + param, e);
        } finally {
            if(in!=null) {
                try {
                    in.close();
                } catch (IOException e) {
                    logger.error(e, e);
                }
            }
        }
        return result.toString();
    }

    public static InputStream sendGetInputStream(String url, String param, Map<String, String> reqHead) {
        StringBuffer result = new StringBuffer();
        InputStream in = null;
        try {
            if (param != null)
                url = url + "?" + param;
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            URLConnection connection = realUrl.openConnection();
            // 设置通用的请求属性
            connection.setRequestProperty("accept", "*/*");
            connection.setRequestProperty("connection", "Keep-Alive");

            if (reqHead != null) {
                for (Map.Entry<String, String> entry : reqHead.entrySet()) {
                    if (StringUtils.isNotBlank(entry.getValue())) {
                        connection.setRequestProperty(entry.getKey(), entry.getValue());
                    }
                }
            }
            // 建立实际的连接
            try {
                connection.connect();
            } catch (Exception e) {
                connection.connect();
            }
            // 定义 BufferedReader输入流来读取URL的响应
            in = connection.getInputStream();
        } catch (Exception e) {
            logger.error("发送GET请求出现异常！" + url + param, e);
        } finally {
        }
        return in;
    }

    /**
     * 向指定 URL 发送POST方法的请求
     *
     * @param url   发送请求的 URL
     * @param param 请求参数，请求参数应该是 name1=value1&name2=value2 的形式。
     * @return 所代表远程资源的响应结果
     */
    public static String sendPost(String url, String param) {
        return sendPost(url, param, charset);
    }

    public static String sendPost(String url, String param, String charset) {
        PrintWriter out = null;
        BufferedReader in = null;
        String result = "";
        try {
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            URLConnection conn = realUrl.openConnection();
            // 设置通用的请求属性
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            conn.setRequestProperty("Referer", url);
            // 发送POST请求必须设置如下两行
            conn.setDoOutput(true);
            conn.setDoInput(true);
            // 获取URLConnection对象对应的输出流
            out = new PrintWriter(conn.getOutputStream());
            // 发送请求参数
            out.print(param);
            // flush输出流的缓冲
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应
            in = new BufferedReader(new InputStreamReader(conn.getInputStream(), charset));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            logger.error("发送POST请求出现异常！" + url + ",param:" + param, e);
        }
        // 使用finally块来关闭输出流、输入流
        finally {
            if(in!=null) {
                try {
                    in.close();
                } catch (IOException e) {
                    logger.error(e, e);
                }
            }
            if(out!=null) {
                out.close();
            }
        }
        return result;
    }

    public static String doPost(String url, String content, String contentType
            , int connTimeOut, int readTimeOut) throws Exception {
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse httpResponse = null;
        String result = "";
        // 创建httpClient实例
        httpClient = HttpClients.createDefault();
        // 创建httpPost远程连接实例
        HttpPost httpPost = new HttpPost(url);
        // 配置请求参数实例
        RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(connTimeOut)// 设置连接主机服务超时时间
                .setConnectionRequestTimeout(connTimeOut)// 设置连接请求超时时间
                .setSocketTimeout(readTimeOut)// 设置读取数据连接超时时间
                .build();
        // 为httpPost实例设置配置
        httpPost.setConfig(requestConfig);
        // 设置请求头
        httpPost.addHeader("Content-Type", contentType);
        // 封装post请求参数
        if (null != content) {
            // 为httpPost设置封装好的请求参数
            byte[] b = content.getBytes(StandardCharsets.UTF_8);
            InputStream is = new ByteArrayInputStream(b, 0, b.length);
            httpPost.setEntity(new InputStreamEntity(is, b.length
                    , ContentType.create(contentType, StandardCharsets.UTF_8)));
        }
        try {
            // httpClient对象执行post请求,并返回响应参数对象
            httpResponse = httpClient.execute(httpPost);

            // 从响应对象中获取响应内容
            HttpEntity entity = httpResponse.getEntity();
            result = EntityUtils.toString(entity);

            if (httpResponse.getStatusLine().getStatusCode() != 200) {
                throw new Exception(result);
            }
        } catch (ClientProtocolException e) {
            throw e;
        } catch (IOException e) {
            throw e;
        } finally {
            // 关闭资源
            if (null != httpResponse) {
                try {
                    httpResponse.close();
                } catch (IOException e) {
                    throw e;
                }
            }
            if (null != httpClient) {
                try {
                    httpClient.close();
                } catch (IOException e) {
                    throw e;
                }
            }
        }
        return result;
    }

    /**
     * 发送get请求
     * @param url 请求地址,返回码不为200时，返回错误信息
     * @return 请求结果
     */
    public static String doGet(String url) throws Exception {
        InputStream inputStream = null;
        InputStreamReader inputStreamReader = null;
        BufferedReader bufferedReader = null;
        String responseResult = "";
        int code;
        // 打开连接
        URL requestUrl = new URL(url);
        HttpURLConnection httpConn = (HttpURLConnection) requestUrl.openConnection();

        // 加入数据
        httpConn.setRequestMethod("GET");
        httpConn.setRequestProperty("Accept", "application/json");
        httpConn.setRequestProperty("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.132 Safari/537.36");
        httpConn.setRequestProperty("Accept-Charset", "utf-8");
        httpConn.setRequestProperty("Content-Type", "application/json;charset=utf-8");
        code = httpConn.getResponseCode();
        System.out.println(code);

        if (code == HttpsURLConnection.HTTP_OK) {
            inputStream = httpConn.getInputStream();
            inputStreamReader = new InputStreamReader(inputStream, "UTF-8");
            bufferedReader = new BufferedReader(inputStreamReader);
            String str = null;
            StringBuffer buffer = new StringBuffer();
            while ((str = bufferedReader.readLine()) != null) {
                buffer.append(str);
            }
            responseResult = buffer.toString();

            //responseResult = responseResult.substring(1, responseResult.lastIndexOf('"'));
            responseResult = responseResult.replaceAll("\\\\", "");
            return responseResult;
        } else {
            InputStream is = httpConn.getErrorStream();    //通过getErrorStream了解错误的详情，因为错误详情也以XML格式返回，因此也可以用JDOM来获取。
            InputStreamReader isr = new InputStreamReader(is, "utf-8");
            BufferedReader in = new BufferedReader(isr);
            String inputLine;
            String responseErrorResult = null;
            while ((inputLine = in.readLine()) != null) {
                responseErrorResult = inputLine;
            }
            in.close();
            logger.info("Http返回状态码为非200：" + responseErrorResult);
            return responseErrorResult;
        }
    }
}
