**项目介绍**
- 
- 

**模块划分**
```csv
 模块名称         模块编码
 cnpc-common      1000
 cnpc-message     1001
 cnpc-auth        1002
 cnpc-task        1003
 
 cnpc-third-im    1110
 cnpc-third-sms   1111
```

**项目模块**
- C00-common      公共模块：通用Util、基础服务接口、基础dao接口、异常、分页等
- C01-message     消息模块：接收业务系统的消息发送请求并做相应处理转发到 im、sms、email等服务中
- D00-third-party 调用第三方模块：im、短信等
- D01-dependency  模块的依赖整合模块：
- D03-deploy      微服务整合部署模块， 各种Application

**项目结构**
```
platform
├─C00-common        公共模块
├─C01-message       消息模块
│    ├─cnpc-message-core        消息-核心模块：
│    │  ├─cnpc.core.message    module 总包名
│    │  │   ├─constant  常量
│    │  │   ├─entity  实体
│    │  │   ├─mapper  映射
│    │  │   ├─module  包含的模块，以及初始化架构模块实现类
│    │  │   ├─mybatis  mybatis 配置类
│    │  │   ├─service  服务实现类
│    │  │   │   ├─data.{表名}  出入参model
│    │  │   ├─utils  工具类
│    │  │   ├─provider.impl  架构基础功能实现类
│    ├─cnpc-message-cloud         消息-api模块：
├─D00-third-party   调用第三方模块
├─D01-dependency    产品依赖整合模块
├─D03-deploy        微服务部署模块
│    ├─deploy-cnpc-auth            认证服务       
│    ├─deploy-cnpc-message          系统服务
│ 
│
```

**项目模块编号(错误码基号)**
- C00-common      1000
- C01-message     1001

*third-party 以2000为基号*
- cnpc-third-im   2001
- cnpc-third-sms  2002

**快速开始：**
```shell
1 导入项目
pom.xml

2 执行
mvn clean install
```

```shell
1 导入项目
D03-deploy/pom.xml

2 执行
mvn clean package
```

<br>

**技术选型：**
- 核心框架：Spring Boot 2.6.7 + Spring Cloud 3.1.1 + Spring 5.3.19
- 持久层框架：MyBatis 3.5
- 缓存中间件：Redis 4.x
- 定时器：Quartz 2.3
- 日志管理：Logback
- 接口文档：Swagger3.x
- 数据库版本管理：Flayway6.x
- 数据校验：JSR303
- 方法构建：Lombook

<br>


**项目架构**
1、springboot + flyway + swagger + alibaba

**项目规范**
一、 命名规范
- 1、遵守java命名规范
- 2、在常量与变量的命名时，表示类型的名词放在词尾

二、结构规范
- 1、每个模块包括 core 和 api 两部分，core模块之间不可依赖

三、表命名规范
- 1、小写
- 2、公用模块表命名  以base_ 为前缀，业务含义的英文单词结尾，多个中间添加 下划线“_”

四、表结构升级规范
- 1、flyway 规范


**技术点**
- 分布式id：  gaussdb 的UUID 数据类型，不合适用于分布式数据库中，可能产生相同UUID，建议业间件平台提供的分布式ID

